package de.lobster.mq.webspheremq;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import java.util.Hashtable;
import javax.jms.Connection;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.directory.InitialDirContext;
import com.ibm.msg.client.jms.JmsConnectionFactory;
import com.ibm.msg.client.jms.JmsDestination;

public class JmsJndiConnect {

	private static String initialContextUrl = "file:/C:/JNDI-Directory";
	private static String connectionFactoryFromJndi = "ConnectMq";
	private static String destinationFromJndi = "SampleQueue";

	private static int timeout = 15000; // Timeout in ms or 15 seconds

	// State
	private static int status = -2;

	private static Connection connection = null;
	private static Session session = null;
	private static Destination destination = null;
	private static MessageConsumer consumer = null;
	private static MessageProducer producer = null;

	@BeforeClass
	public static void initTest() throws Exception {
		try {		
		// Instantiate the initial context
		String contextFactory = "com.sun.jndi.fscontext.RefFSContextFactory";
		Hashtable<String, String> environment = new Hashtable<String, String>();
		environment.put(Context.INITIAL_CONTEXT_FACTORY, contextFactory);
		environment.put(Context.PROVIDER_URL, initialContextUrl);
		Context context = new InitialDirContext(environment);
		System.out.println("Initial context found!");

		// Lookup the connection factory
		JmsConnectionFactory cf = (JmsConnectionFactory) context.lookup(connectionFactoryFromJndi);

		// Lookup the destination
		destination = (JmsDestination) context.lookup(destinationFromJndi);

		// Create JMS objects
		connection = cf.createConnection();
		session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
		consumer = session.createConsumer(destination);
		producer = session.createProducer(destination);
		// Start the connection
		connection.start();
		}
	    catch (NamingException ne) {
	        System.out.println("The initial context could not be instantiated, or the lookup failed.");
	        recordFailure(ne);
	      }
	}

	@Test
	public void writeMessage() throws JMSException {
		Message message = session.createTextMessage("jUnitTest");
		producer.send(message);
		System.out.println("Message sent successfully...");
	}

	@Test
	public void readMessage() throws JMSException {

		try {
			// And, receive the message
			Message message = consumer.receive(timeout);
			if (message != null) {
				System.out.println("Received message:\n" + message);
			} else {
				System.out.println("No message received!\n");
				recordFailure(null);
			}

			recordSuccess();
		} catch (JMSException jmsex) {
			recordFailure(jmsex);
		} finally {
			if (consumer != null) {
				try {
					consumer.close();
				} catch (JMSException jmsex) {
					System.out.println("Consumer could not be closed.");
					recordFailure(jmsex);
				}
			}

			if (session != null) {
				try {
					session.close();
				} catch (JMSException jmsex) {
					System.out.println("Session could not be closed.");
					recordFailure(jmsex);
				}
			}

			if (connection != null) {
				try {
					connection.close();
				} catch (JMSException jmsex) {
					System.out.println("Connection could not be closed.");
					recordFailure(jmsex);
				}
			}
		}
	}

	@AfterClass
	public static void destroyTest() throws Exception {
	      if (connection != null) {
	          try {
	            connection.close();
	          }
	          catch (JMSException jmsex) {
	            System.out.println("Connection could not be closed.");
	            recordFailure(jmsex);
	          }
	        }
	      
			if (status<0) {
				throw new Exception("Test nicht erfolgreich");
			}
			
	}
	

	/**
	 * Process a JMSException and any associated inner exceptions.
	 * 
	 * @param jmsex
	 */
	private static void processJMSException(JMSException jmsex) {
		System.out.println(jmsex);
		Throwable innerException = jmsex.getLinkedException();
		if (innerException != null) {
			System.out.println("Inner exception(s):");
		}
		while (innerException != null) {
			System.out.println(innerException);
			innerException = innerException.getCause();
		}
		return;
	}

	/**
	 * Record this run as successful.
	 */
	private static void recordSuccess() {
		System.out.println("SUCCESS");
		status = 0;
		return;
	}

	/**
	 * Record this run as failure.
	 * 
	 * @param ex
	 */
	private static void recordFailure(Exception ex) {
		if (ex != null) {
			if (ex instanceof JMSException) {
				processJMSException((JMSException) ex);
			} else {
				System.out.println(ex);
			}
		}
		System.out.println("FAILURE");
		status = -1;
		return;
	}


}
